
/* 
 * file_sel.c --a single pushbutton, when selected, creates a
 * file selection dialog that displays a list of all the writable
 * files in the directory described by the XmNmask of the dialog.
 * This program demonstrates how to use the XmNfileSearchProc for
 * file selection dialog widgets.
 *
 */

#ifndef USE_ATHENA

#define index strchr

#include "unix.h"
#include "x.h"
#include "xm.h"
#include "off.h"
#include "externs.h"

#define OPEN_HELP    401
#define NEW_HELP    402

/* XmStringCharSet charset = XmFONTLIST_DEFAULT_TAG; */
XmStringCharSet charset = XmSTRING_DEFAULT_CHARSET;

/* routined to determine if a file is accessible, a directory,
 * or writable.  Return -1 on all errors or if the file is not
 * writable.  Return 0 if it's a directory or 1 if it's a plain
 * writable file.
 */
int
is_writable( file, pattern )
char *file, *pattern;
{
    char   *ptr;
    struct  stat s_buf;
    int     lf = strlen(file);

    /* if file can't be accessed ( via stat(  ) ) return. */
    if ( stat( file, &s_buf ) == -1 )
        return -1;
    else if ( ( s_buf.st_mode & S_IFMT ) == S_IFDIR )
        return 0; /* a directory */
    else if ( ( s_buf.st_mode & S_IEXEC ) == S_IEXEC )
        return -1; /* a executable */
    else if ( !( s_buf.st_mode & S_IFREG ) || access( file, W_OK ) == -1 )
        /* not a normal file or it is not writable */
        return -1;
    else if( ( file[lf-1] != pattern[4] ||
               file[lf-2] != pattern[3] ||
               file[lf-3] != pattern[2] ||
               file[lf-4] != pattern[1] ) )
        return -1;
#if( 0 ) 
    else if( (ptr = index( file, '.' )) != (char *)NULL &&
        ptr++ &&
        strcmp( ptr, "aoff" ) )
        return -1;
    else if( (ptr = index( file, 'a' )) != (char *)NULL &&
        strcmp( ptr, "aoff" ) )
        return -1;
#endif		/*  */
    /* legitimate file */
    return 1;
}


/* do_search(  ) -- scan a directory and report only those files that
 * are writable.  Here, we let the shell expand the ( possible )
 * wildcards and return a directory listing by using popen(  ).
 * A *real* application should -not- do this; it should use the
 * system's directory routines: opendir(  ), readdir(  ) and closedir(  ).
 */
void
do_search( fs, cbs )
Widget fs; /* file selection box widget */
XmFileSelectionBoxCallbackStruct *cbs;
{
    char          *mask, buf[BUFSIZ], *p;
    XmString       names[256]; /* maximum of 256 files in dir */
    int            i = 0;
    FILE          *pp, *popen(  );
    extern char   *pattern;

    if ( !XmStringGetLtoR( cbs->mask, charset, &mask ) )
        return; /* can't do anything */

    sprintf( buf, "/bin/ls %s", mask );
    XtFree( mask );
    /* let the shell read the directory and expand the filenames */
    if ( !( pp = popen( buf, "r" ) ) )
        return;
    /* read output from popen(  ) -- this will be the list of files */
    while ( fgets( buf, sizeof buf, pp ) ) {
        if ( p = index( buf, '\n' ) )
            *p = 0;
        /* only list files that are writable and not directories */
        if ( is_writable( buf, pattern ) == 1 &&
            ( names[i] = XmStringCreateSimple( buf ) ) )
            i++;
    }
    pclose( pp );
    if ( i ) {
        XtVaSetValues( fs,
            XmNfileListItems,      names,
            XmNfileListItemCount,  i,
            XmNdirSpec,            names[0],
            XmNlistUpdated,        True,
            NULL );
        while ( i > 0 )
            XmStringFree( names[--i] );
    } else
        XtVaSetValues( fs,
            XmNfileListItems,      NULL,
            XmNfileListItemCount,  0,
            XmNlistUpdated,        True,
            NULL );
}

#endif	/* USE_ATHENA */
